package uk.co.mrdw.photostopwatch;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.util.Pair;

/**
 * Holds data relating to a race
 * 
 * @author Mike Warren 21 Aug 2012
 * 
 */
public class RaceData {
	private static final String TAG = "RaceData";

	public static final String TIMES = "uk.co.mrdw.photostopwatch.TIMES";
	public static final String PHOTO_CHECK = "uk.co.mrdw.photostopwatch.PHOTO_CHECK";
	public static final String RACE_NAME = "uk.co.mrdw.photostopwatch.RACE_NAME";
	public static final String RACE_IS_RUNNING = "uk.co.mrdw.photostopwatch.RACE_IS_RUNNING";
	public static final String CHRONO_BASE = "uk.co.mrdw.photostopwatch.CHRONOMETER_BASE";

	private static final DecimalFormat df2 = new DecimalFormat("00");
	private static final DecimalFormat df3 = new DecimalFormat("000");

	private static final String DEFAULT_RACE_NAME = "blank";

	private static final String START_TIME_FORMAT_3 = "d MMM yyyy HH:mm:ss.SSS";

	private static int decimalPlaces = 1;

	private long chronometerBase = 0;
	private boolean chronometerRunning = false;
	private String raceName = DEFAULT_RACE_NAME;
	private File raceFolder = null;
	private ArrayList<Pair<Long, Boolean>> results = new ArrayList<Pair<Long, Boolean>>();

	public static RaceData loadFromArchive(String raceName) {
		File raceTimesFile = Utils.getRaceTimesFile(raceName);
		if (!raceTimesFile.exists()) {
			return null;
		}
		RaceData raceData = new RaceData();
		raceData.setRaceName(raceName);

		BufferedReader br =null;
		try {
			br = new BufferedReader(new FileReader(raceTimesFile));
			while (br.ready()) {
				String line = br.readLine();// TODO this code should be in same place as code that writes out this
				String[] lineValues = line.split(",");
				Long time = 0l;
				if (lineValues.length > 0) {
					time = Long.valueOf(lineValues[0]);
				}
				Boolean photoCheck = false;
				if (lineValues.length > 1) {
					photoCheck = Boolean.valueOf(lineValues[1]);
				}
				raceData.getResults().add(new Pair<Long, Boolean>(time, photoCheck));
			}

		} catch (IOException e) {
			Log.e(TAG, "Unable to read in race " + raceName, e);
			return null;
		}
		finally{
			if(br!=null){
				try {
					br.close();
				} catch (IOException e) {
					Log.e(TAG, "Unable to read in race, failed closing reader. " + raceName, e);
					return null;
				}
			}
		}
		return raceData;
	}

	public RaceData() {
		this(false, DEFAULT_RACE_NAME, new ArrayList<Pair<Long, Boolean>>());
	}

	public RaceData(boolean chronometerRunning, String raceName, ArrayList<Pair<Long, Boolean>> times) {
		super();
		this.chronometerRunning = chronometerRunning;
		this.raceName = raceName;
		this.results = times;
	}

	// public RaceData(Intent intent) {
	// populateFromIntent(intent);
	// }

	/**
	 * Synchronized method, so can be relied on to wait until folder created if necesary before returning
	 * 
	 * @return
	 */
	public synchronized File getRaceFolder() {
		if (raceFolder == null) {
			raceFolder = getOrCreateRaceFolder();
		}
		return raceFolder;
	}

	public synchronized void reset() {
		raceFolder = null;
		chronometerRunning = false;
		raceName = DEFAULT_RACE_NAME;
		chronometerBase = 0;
		results.clear();
	}

	public boolean isReset() {
		return chronometerRunning == false && raceName.equals(DEFAULT_RACE_NAME) && results.size() == 0;
	}

	private File getOrCreateRaceFolder() {
		File archiveDir = Utils.getRaceFolderWithoutCreating(raceName);
		// Create the storage directory if it does not exist
		if (!archiveDir.exists()) {
			if (!archiveDir.mkdirs()) {
				Log.e(TAG, "failed to create directory " + archiveDir.getAbsolutePath());
				return archiveDir;
			}
			File imagesFolder = Utils.getRaceImagesFolderWithoutCreating(raceName);
			if (!imagesFolder.exists() && !imagesFolder.mkdirs()) {
				Log.e(TAG, "failed to create images directory " + imagesFolder.getAbsolutePath());
				return archiveDir;
			}
		}
		return archiveDir;
	}

	// TODO delete
	public void populateIntent(Intent intent) {
		intent.putExtra(RACE_NAME, getRaceName());
		intent.putExtra(TIMES, getTimesArray());
		intent.putExtra(PHOTO_CHECK, getPhotoChecksArray());
		intent.putExtra(RACE_IS_RUNNING, isChronometerRunning());
		intent.putExtra(CHRONO_BASE, getChronometerBase());
	}

	// public void populateFromIntent(Intent intent){
	// raceName = intent.getStringExtra(RACE_NAME);
	//
	// if(raceName.equals( TimerActivity.getCurrentRace().raceName)){
	// results = new ArrayList<Pair<Long,Boolean>>();
	// results.addAll(TimerActivity.getCurrentRace().getResults());
	// chronometerBase = TimerActivity.getCurrentRace().getChronometerBase();
	// }
	//
	// long[] timesArray = intent.getLongArrayExtra(TIMES);
	// boolean[] photosArray = intent.getBooleanArrayExtra(PHOTO_CHECK);
	// populateResults(timesArray,photosArray);
	// this.chronometerRunning = intent.getBooleanExtra(RACE_IS_RUNNING, false);
	// this.chronometerBase = intent.getLongExtra(CHRONO_BASE, 0);
	// }

	public void populateBundle(Bundle savedInstanceState) {
		savedInstanceState.putString(RACE_NAME, getRaceName());
		savedInstanceState.putLongArray(TIMES, getTimesArray());
		savedInstanceState.putBooleanArray(PHOTO_CHECK, getPhotoChecksArray());
		savedInstanceState.putBoolean(RACE_IS_RUNNING, isChronometerRunning());
		savedInstanceState.putLong(CHRONO_BASE, getChronometerBase());
	}

	public void populateFromBunndle(Bundle savedInstanceState) {
		this.raceName = savedInstanceState.getString(RACE_NAME);
		long[] timesArray = savedInstanceState.getLongArray(TIMES);
		boolean[] photosArray = savedInstanceState.getBooleanArray(PHOTO_CHECK);
		populateResults(timesArray, photosArray);
		this.chronometerRunning = savedInstanceState.getBoolean(RACE_IS_RUNNING, false);
		this.chronometerBase = savedInstanceState.getLong(CHRONO_BASE);
	}

	public String getStartTimeString() {
		Date startTime = getStartTimeDate();
		String ret = null;

		if (startTime != null) {
			ret = new SimpleDateFormat(START_TIME_FORMAT_3).format(startTime);
			if(decimalPlaces==0){
				ret = ret.substring(0,ret.length()-(4));// get rid of decimal place							
			}
			else if(decimalPlaces<3){
				ret = ret.substring(0,ret.length()-(3-decimalPlaces));				
			}
		}

		return ret;
	}

	public Date getStartTimeDate() {
		if (results.size() == 0) {
			return null;
		}
		Date startTime = new Date(results.get(0).first);
		return startTime;
	}

	/**
	 * Return time for race position (i.e. starting at 1, position 0 being the start time in millis )
	 * 
	 * @param position
	 * @return time in milliseconds
	 */
	public Long getTimeForPosition(int position) {
		return results.get(position).first;
	}

	/**
	 * Get the date time for the race position - i.e. if race started at 9:00am 29/8/2012 and time is 10 minutes this will give 9:10am 29/8/2012
	 * 
	 * @param position
	 * @return
	 */
	public Date getDateForPostion(int position) {
		return new Date(getTimeForPosition(position) + results.get(0).first);
	}


	/**
	 * Is there a photo for this position ?
	 * @param position
	 * @return
	 */
	public boolean isPhotoCheckForPosition(int position){
		return results.get(position).second;
	}
	
	/**
	 * Return time for race position (i.e. starting at 1, position 0 being the start time in millis )
	 * 
	 * @param position
	 * @return time in milliseconds
	 */
	public String getTimeStringForPosition(int position) {
		long timeInMillis = results.get(position).first;
		return RaceData.getTimeString(timeInMillis);
	}

	public static String getTimeString(long timeInMillis) {
		long millis = timeInMillis % 1000;
		long secs = ((timeInMillis - millis) / 1000) % 60;
		long mins = ((timeInMillis - millis - secs) / (1000 * 60)) % 60;
		long hours = ((timeInMillis - millis - secs - mins) / (1000 * 60 * 60)) % 60;
		// df2.format(millis);
		String millisString = df3.format(millis).substring(0, decimalPlaces);

		return (hours != 0 ? df2.format(hours) + ":" : "") + df2.format(mins) + ":" + df2.format(secs) + (decimalPlaces != 0 ? "." : "") + millisString;

	}

	public static String getParkrunTimeString(long timeInMillis) {
		long millis = timeInMillis % 1000;
		long secs = ((timeInMillis - millis) / 1000) % 60;
		long mins = ((timeInMillis - millis - secs) / (1000 * 60)) % 60;
		long hours = ((timeInMillis - millis - secs - mins) / (1000 * 60 * 60)) % 60;
		// df2.format(millis);
		return df2.format(hours) + ":" + df2.format(mins) + ":" + df2.format(secs)
		// + ":"+df3.format(millis) // not displaying millis for now
		;

	}

	public long getChronometerBase() {
		return chronometerBase;
	}

	public void setChronometerBase(long chronometerBase) {
		this.chronometerBase = chronometerBase;
	}

	public boolean isChronometerRunning() {
		return chronometerRunning;
	}

	public void setChronometerRunning(boolean chronometerRunning) {
		this.chronometerRunning = chronometerRunning;
	}

	public String getRaceName() {
		return raceName;
	}

	public void setRaceName(String raceName) {
		this.raceName = raceName;
	}

	public ArrayList<Pair<Long, Boolean>> getResults() {
		return results;
	}

	public long[] getTimesArray() {
		long[] array = new long[results.size()];
		for (int i = 0; i < array.length; i++) {
			array[i] = results.get(i).first;
		}
		return array;
	}
	
	public boolean[] getPhotoChecksArray() {
		boolean[] array = new boolean[results.size()];
		for (int i = 0; i < array.length; i++) {
			array[i] = results.get(i).second;
		}
		return array;
	}

	private void populateResults(long[] timesArray, boolean[] photoChecksArray) {
		this.results = new ArrayList<Pair<Long, Boolean>>();
		if (timesArray == null || photoChecksArray == null || timesArray.length != photoChecksArray.length) {
			Log.e(TAG, "Couldn't match times and photo checks ? " + this.raceName + " " + this.chronometerRunning);
			return;
		}
		for (int i = 0; i < timesArray.length; i++) {
			results.add(new Pair<Long, Boolean>(timesArray[i], photoChecksArray[i]));
		}

	}

	public static int getDecimalPlaces() {
		return decimalPlaces;
	}

	public static void setDecimalPlaces(int decimalPlaces) {
		RaceData.decimalPlaces = decimalPlaces;
	}

	
	/**
	 * Checks that expected files that need to be generated have been
	 * @param raceName
	 * @return
	 */
	public static boolean isRaceReadyForReview(String raceName){
		File raceTimesHtmlFile = Utils.getRaceTimesHtmlFile(raceName);
		File raceTimesAndPhotosHtmlFile = Utils.getRaceTimesAndPhotosHtmlFile(raceName);
		File raceTimesFile = Utils.getRaceTimesFile(raceName);
		if(raceTimesFile.exists()){
			long lastUpdate = raceTimesFile.lastModified();
			if(	raceTimesHtmlFile.exists() && raceTimesHtmlFile.lastModified()>lastUpdate 
				&& raceTimesAndPhotosHtmlFile.exists() && raceTimesAndPhotosHtmlFile.lastModified()>lastUpdate ){
				return true;
			}
		}
		return false; 
	}

	public static boolean isPhotosNeedRotating(String raceName){
		return false; // TODO
	}

	
	// public void setTimesArray(long[] timesArray) {
	// this.times = new ArrayList<Pair<Long,Boolean>> ();
	// if(timesArray!=null){
	// for(long time: timesArray){
	// times.add(time);
	// }
	// }
	// }
	// public void setTimes(ArrayList<Long> times) {
	// this.times = times;
	// }
	// public ArrayList<String> getTimeStrings() {
	// return null; // TODO
	// }
	// public void setTimesStrings(ArrayList<String> times) {
	// // TODO if required
	// // this.times = times;
	// }
}
