package uk.co.mrdw.photostopwatch;

import android.content.SharedPreferences;
import android.util.Log;

/**
 * Holds settings, such as time format, left/right handed, headset enabled to allow them to be 
 * edited, and saved to app's preferences
 * 
 * @author Mike Warren
 * 11 Sep 2012
 *
 */
public class Settings implements Cloneable {
	private static final String TAG = "Settings";

	public static final int PIC_HEIGHT_SMALL = 300;
	public static final int PIC_HEIGHT_LARGE = 800;
	
	private static final int MAX_DPS= 3;
	private static final int MIN_DPS= 0;

	public static final int ROTATE_IMAGES_NO = 0;
	public static final int ROTATE_IMAGES_ALWAYS = 1;
	public static final int ROTATE_IMAGES_AUTO = 2;
	
	private static final boolean LEFT_HANDED_DEFAULT = false;
	private static final boolean HEADSET_ENABLED_DEFAULT = false;
	private static final boolean VOLUME_KEYS_ENABLED_DEFAULT = false;
	private static final boolean CHRONO_SHOW_TENTHS_DEFAULT = false;
	private static final int TIME_DECIMAL_PLACES_DEFAULT = 1;
	private static final int MIN_PICTURE_HEIGHT_DEFAULT = PIC_HEIGHT_SMALL;
	private static final int ROTATE_IMAGES_DEFAULT = ROTATE_IMAGES_AUTO;
	private static final boolean ROTATE_IMAGES_KEEP_ORIG_DEFAULT = true;
	
	private boolean leftHanded = LEFT_HANDED_DEFAULT;
	private boolean headsetEnabled = HEADSET_ENABLED_DEFAULT;
	private boolean volumeKeysEnabled = VOLUME_KEYS_ENABLED_DEFAULT;
	private boolean chronoShowTenths = CHRONO_SHOW_TENTHS_DEFAULT;
	private int timeDecimalPlaces = TIME_DECIMAL_PLACES_DEFAULT;
	private int minPictureHeight = MIN_PICTURE_HEIGHT_DEFAULT;
	private int rotateImages = ROTATE_IMAGES_DEFAULT;
	private boolean rotateImagesKeepOrig = ROTATE_IMAGES_KEEP_ORIG_DEFAULT;
	
	public void loadFromSharedPreferences(SharedPreferences sharedPrefs){
		setLeftHanded( sharedPrefs.getBoolean(Utils.PREFS_LEFT_HANDED, LEFT_HANDED_DEFAULT) );
		setHeadsetEnabled( sharedPrefs.getBoolean(Utils.PREFS_HEADSET_CONTROL, HEADSET_ENABLED_DEFAULT) );
		setVolumeKeysEnabled(sharedPrefs.getBoolean(Utils.PREFS_VOLUME_CONTROL, VOLUME_KEYS_ENABLED_DEFAULT));
		setChronoShowTenths( sharedPrefs.getBoolean(Utils.PREFS_CHRONO_SHOW_TENTHS, CHRONO_SHOW_TENTHS_DEFAULT) );
		setTimeDecimalPlaces( sharedPrefs.getInt(Utils.PREFS_TIME_DPS, TIME_DECIMAL_PLACES_DEFAULT) );
		setMinPictureHeight( sharedPrefs.getInt(Utils.PREFS_MIN_PICTURE_HEIGHT,  MIN_PICTURE_HEIGHT_DEFAULT));
		setRotateImages( sharedPrefs.getInt(Utils.PREFS_ROTATE_IMAGES, ROTATE_IMAGES_DEFAULT));
		setRotateImagesKeepOrig( sharedPrefs.getBoolean(Utils.PREFS_ROTATE_IMAGES_KEEP_ORIG, ROTATE_IMAGES_KEEP_ORIG_DEFAULT));
	}
		
	public void saveToSharedPreferences(SharedPreferences sharedPrefs){
		SharedPreferences.Editor editor = sharedPrefs.edit();
		editor.putBoolean(Utils.PREFS_LEFT_HANDED, isLeftHanded());
		editor.putBoolean(Utils.PREFS_HEADSET_CONTROL, isHeadsetEnabled());
		editor.putBoolean(Utils.PREFS_VOLUME_CONTROL, isVolumeKeysEnabled());
		editor.putBoolean(Utils.PREFS_CHRONO_SHOW_TENTHS, isChronoShowTenths());
		editor.putInt(Utils.PREFS_TIME_DPS, getTimeDecimalPlaces() );
		editor.putInt(Utils.PREFS_MIN_PICTURE_HEIGHT, getMinPictureHeight());
		editor.putInt(Utils.PREFS_ROTATE_IMAGES, getRotateImages());
		editor.putBoolean(Utils.PREFS_ROTATE_IMAGES_KEEP_ORIG, isRotateImagesKeepOrig());
		editor.commit();		
	}
	
	public Settings copy(){
		try {
			return (Settings)this.clone();// all primitive types so ok - will need to change if adding reference types
		} catch (CloneNotSupportedException e) {
			Log.e(TAG,"",e);
		}
		return null;
	}
	
	public boolean isLeftHanded() {
		return leftHanded;
	}
	public void setLeftHanded(boolean leftHanded) {
		this.leftHanded = leftHanded;
	}
	public boolean isHeadsetEnabled() {
		return headsetEnabled;
	}
	public void setHeadsetEnabled(boolean headsetEnabled) {
		this.headsetEnabled = headsetEnabled;
	}
	public boolean isVolumeKeysEnabled() {
		return volumeKeysEnabled;
	}
	public void setVolumeKeysEnabled(boolean volumeKeysEnabled) {
		this.volumeKeysEnabled = volumeKeysEnabled;
	}

	public boolean isChronoShowTenths() {
		return chronoShowTenths;
	}

	public void setChronoShowTenths(boolean chronoShowTenths) {
		this.chronoShowTenths = chronoShowTenths;
	}

	public int getTimeDecimalPlaces() {
		return timeDecimalPlaces;
	}
	public void setTimeDecimalPlaces(int timeDecimalPlaces) {
		this.timeDecimalPlaces =  Math.min(Math.max(timeDecimalPlaces,MIN_DPS),MAX_DPS);
	}

	public int getMinPictureHeight() {
		return minPictureHeight;
	}

	public void setMinPictureHeight(int minPictureHeight) {
		this.minPictureHeight = minPictureHeight;
	}

	public int getRotateImages() {
		return rotateImages;
	}

	public void setRotateImages(int rotateImages) {
		this.rotateImages = rotateImages;
	}

	public boolean isRotateImagesKeepOrig() {
		return rotateImagesKeepOrig;
	}

	public void setRotateImagesKeepOrig(boolean rotateImagesKeepOrig) {
		this.rotateImagesKeepOrig = rotateImagesKeepOrig;
	}
	
}
