package uk.co.mrdw.photostopwatch;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintStream;
import java.io.RandomAccessFile;
import java.net.URI;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.BitmapFactory.Options;
import android.graphics.Matrix;
import android.hardware.Camera;
import android.os.Build;
import android.os.Environment;
import android.util.Log;
import android.view.Surface;

public final class Utils {
	private static final String TAG = "Utils";

	private static final String WINDOWS_NEWLINE = "\r\n";

	public static final String PREFS_NAME = "PhotostopwatchPrefs";
	public static final String PREFS_TIME_DPS = "PREFS_TIME_DPS";
	public static final String PREFS_MIN_PICTURE_HEIGHT = "PREFS_MIN_PICTURE_HEIGHT";
	public static final String PREFS_LEFT_HANDED = "PREFS_LEFT_HANDED";
	public static final String PREFS_HEADSET_CONTROL = "PREFS_HEADSET_CONTROL";
	public static final String PREFS_VOLUME_CONTROL = "PREFS_VOLUME_CONTROL";
	public static final String PREFS_CHRONO_SHOW_TENTHS = "PREFS_CHRONO_SHOW_TENTHS";
	public static final String PREFS_ROTATE_IMAGES = "PREFS_ROTATE_IMAGES";
	public static final String PREFS_ROTATE_IMAGES_KEEP_ORIG = "PREFS_ROTATE_IMAGES_KEEP_ORIG";
	public static final String PREFS_VERSION_NAME = "PREFS_VERSION_NAME";

	private static final String APPLICATION_FOLDER = "PhotoStopwatch";
	private static final String IMAGES_FOLDER = "images";
	public static final String RACE_TIMES_TEXT_FILE_NAME = "racetimes.txt";
	public static final String RACE_TIMES_AND_PHOTOS_HTML_FILE_NAME = "racephotos.html";
	public static final String RACE_TIMES_HTML_FILE_NAME = "racetimes.html";

	public static final int JPEG_QUALITY = 30;

	public static final String ANDROID_APP_URL = "https://play.google.com/store/apps/details?id=uk.co.mrdw.photostopwatch";

	private static final SimpleDateFormat parkrunDateTimeFormat = new SimpleDateFormat(
			"dd/MM/yyyy HH:mm:ss",Locale.US);

	private static File racesFolder = null;// requires initialisation by
											// initialiseStorageDirPath

	private static String versionName = "?";

	// check sd card is available ...
	// http://stackoverflow.com/questions/902089/how-to-tell-if-the-sdcard-is-mounted-in-android
	static public boolean hasWritableStorage() {
		return hasStorage(true);
	}

	/**
	 * Need to do this because need a context to call
	 * context.getExternalFilesDir Rather than have every path related util
	 * method require a contect, initialise the folder here in the onCreate of
	 * each activity ... until I work out a nicer way to do this.
	 * 
	 * @param context
	 */
	static public void initialiseStorageDirPath(Context context) {
		racesFolder = getRacesFolder(context);
	}

	static public boolean hasStorage(boolean requireWriteAccess) {
		String state = Environment.getExternalStorageState();
		Log.d(TAG, "storage state is " + state);

		if (Environment.MEDIA_MOUNTED.equals(state)) {
			if (requireWriteAccess) {
				boolean writable = checkFsWritable();
				Log.v(TAG, "storage writable is " + writable);
				return writable;
			} else {
				return true;
			}
		} else if (!requireWriteAccess
				&& Environment.MEDIA_MOUNTED_READ_ONLY.equals(state)) {
			return true;
		}
		return false;
	}

	// http://grepcode.com/file/repository.grepcode.com/java/ext/com.google.android/android-apps/2.1_r2/com/android/camera/ImageManager.java#ImageManager.checkFsWritable%28%29
	private static boolean checkFsWritable() {
		// Create a temporary file to see whether a volume is really writeable.
		// It's important not to put it in the root directory which may have a
		// limit on the number of files.
		File directory = getRacesFolder();
		if (!directory.isDirectory()) {
			if (!directory.mkdirs()) {
				return false;
			}
		}
		File f = new File(directory, ".probe");
		try {
			// Remove stale file if any
			if (f.exists()) {
				f.delete();
			}
			if (!f.createNewFile()) {
				return false;
			}
			f.delete();
			return true;
		} catch (IOException ex) {
			return false;
		}
	}

	public static File[] getArchiveFolders() {
		File archiveDirectory = getRacesFolder();
		File[] archives = archiveDirectory.listFiles(new FileFilter() {

			public boolean accept(File pathname) {
				return pathname.isDirectory();
			}

		});

		Arrays.sort(archives);
		return archives;
	}

	/**
	 * Roate the images for a given race. Will backup original images, then
	 * resample and rotate back to original filenames
	 * 
	 * @param raceName
	 * @param checkIfNeeded
	 *            - if true will only rotate if the width>length, otherwise it
	 *            assumes rotation not needed
	 * @param progressIndicator
	 */
	public static void rotateImages(String raceName, boolean checkIfNeeded,
			PercentHolder progressIndicator) {
		File raceImagesFolder = getRaceImagesFolderWithoutCreating(raceName);
		File[] images = raceImagesFolder.listFiles(new FilenameFilter() {
			
			public boolean accept(File dir, String name) {
				if (name.startsWith("img_") && name.endsWith(".jpg")) {
					return true;
				}
				return false;
			}

		});

		if(images==null){
			return;
		}
		if (checkIfNeeded) {
			images = listImagesThatNeedRotating(images);
		}

		try {
			Matrix matrix = new Matrix();
			matrix.postRotate(90);
			Options options = new Options();
			int progress =0;
			for (File image : images) {
				options.inJustDecodeBounds = true;

				File backup = new File(image.getParentFile(), "orig_"
						+ image.getName());
				OutputStream backupOutputStream = null;
				OutputStream rotatedImageOs = null;
				try {
					backupOutputStream = new FileOutputStream(backup);
					copy(image, backupOutputStream);
					BitmapFactory.decodeFile(backup.getAbsolutePath(), options);
					options.inSampleSize = calculateInSampleSize(options, 300,
							300);
					options.inJustDecodeBounds = false;
					Bitmap bitmapToSave = BitmapFactory.decodeFile(
							backup.getAbsolutePath(), options);
					bitmapToSave = Bitmap.createBitmap(bitmapToSave, 0, 0,
							bitmapToSave.getWidth(), bitmapToSave.getHeight(),
							matrix, true);

					delete(image);
					rotatedImageOs = new FileOutputStream(image);
					bitmapToSave.compress(Bitmap.CompressFormat.JPEG,
							JPEG_QUALITY, rotatedImageOs);
					progress ++;
					progressIndicator.setPercent( progress*100/images.length);
				} finally {
					if (backupOutputStream != null) {
						backupOutputStream.close();
					}
					if (rotatedImageOs != null) {
						rotatedImageOs.close();
					}
				}

			}
		} catch (IOException ioe) {
			Log.e(TAG, "Rotating images failed ", ioe);
		}

	}

	private static File[] listImagesThatNeedRotating(File[] images) {
		List<File> imagesThatNeedRotating = new ArrayList<File>();
		Options options = new Options();
		options.inJustDecodeBounds = true;
		for (File image : images) {
			BitmapFactory.decodeFile(image.getAbsolutePath(), options);
			if (options.outWidth > options.outHeight) {
				imagesThatNeedRotating.add(image);
			}
		}

		return imagesThatNeedRotating.toArray(new File[] {});
	}

	// ----------------------------------------------------------------------------------------------------
	// from here
	// http://developer.android.com/training/displaying-bitmaps/load-bitmap.html
	// ----------------------------------------------------------------------------------------------------
	public static int calculateInSampleSize(BitmapFactory.Options options,
			int reqWidth, int reqHeight) {
		// Raw height and width of image
		final int height = options.outHeight;
		final int width = options.outWidth;
		int inSampleSize = 1;

		if (height > reqHeight || width > reqWidth) {
			if (width > height) {
				inSampleSize = Math.round((float) height / (float) reqHeight);
			} else {
				inSampleSize = Math.round((float) width / (float) reqWidth);
			}
		}
		return inSampleSize;
	}

//
//	public static void moveFile(File src, File dst) throws IOException {
//		Log.d(TAG,
//				"Moving file " + src.getAbsolutePath() + " to "
//						+ dst.getAbsolutePath());
//		if (dst.exists()) {
//			throw new IOException(
//					"Destintation already exists, failed to move file "
//							+ src.getAbsolutePath() + " to "
//							+ dst.getAbsolutePath());
//		}
//		if (src.isDirectory()) {
//			throw new IOException("(Directory not file) failed to move file "
//					+ src.getAbsolutePath() + " to " + dst.getAbsolutePath());
//		}
//		dst.getParentFile().mkdirs();
//		InputStream in = new FileInputStream(src);
//		OutputStream out = new FileOutputStream(dst);
//
//		// Transfer bytes from in to out
//		byte[] buf = new byte[1024];
//		int len;
//		while ((len = in.read(buf)) > 0) {
//			out.write(buf, 0, len);
//		}
//		in.close();
//		out.close();
//		if (dst.exists()) {
//			boolean deleted = src.delete();
//			if (!deleted) {
//				throw new IOException(
//						"failed to delete src after copying file "
//								+ src.getAbsolutePath() + " to "
//								+ dst.getAbsolutePath());
//			}
//		}
//
//	}

	public static File getRacesFolder() {
		return racesFolder;
	}

	@SuppressLint("NewApi")
	private static File getRacesFolder(Context context) {
		File racesFolder;
		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.FROYO) {
			// only api >= 8
			racesFolder = new File(context.getExternalFilesDir(null),
					APPLICATION_FOLDER);
		} else {
			racesFolder = new File(Environment.getExternalStorageDirectory(),
					APPLICATION_FOLDER);
		}
		return racesFolder;
	}


	public static boolean delete(File f) {
		if (f.isDirectory()) {
			for (File c : f.listFiles()) {
				delete(c);
			}
		}
		if (!f.delete()) {
			Log.e(TAG, "Failed to delete file: " + f);
			return false;
		}
		return true;
	}

	public static String getImageFileName(long time) {
		return "img_" + time + ".jpg";
	}

	public static File getRaceFolderWithoutCreating(String raceName) {
		File archiveDir = new File(Utils.getRacesFolder(), raceName);
		return archiveDir;
	}

	public static File getRaceImagesFolderWithoutCreating(String raceName) {
		File archiveDir = new File(getRaceFolderWithoutCreating(raceName),
				IMAGES_FOLDER);
		return archiveDir;
	}

	public static File getImageFile(RaceData race, long time) {
		// To be safe, you should check that the SDCard is mounted
		// using Environment.getExternalStorageState() before doing this.

		File mediaStorageDir = new File(race.getRaceFolder(), IMAGES_FOLDER);
		Log.i(TAG, "mediaStorageDir path: " + mediaStorageDir.getAbsolutePath());

		File mediaFile = new File(mediaStorageDir, getImageFileName(time));
		Log.i(TAG, "File path: " + mediaFile.getAbsolutePath());
		return mediaFile;
	}

	public static File getRaceTimesFile(RaceData race) {
		File raceTimesFile = new File(race.getRaceFolder(),
				RACE_TIMES_TEXT_FILE_NAME);
		return raceTimesFile;
	}

	public static File getRaceTimesFile(String raceName) {
		File raceTimesFile = new File(getRaceFolderWithoutCreating(raceName),
				RACE_TIMES_TEXT_FILE_NAME);
		return raceTimesFile;
	}

	public static File getRaceTimesHtmlFile(RaceData race) {
		File raceTimesFile = new File(race.getRaceFolder(),
				RACE_TIMES_HTML_FILE_NAME);
		return raceTimesFile;
	}

	public static File getRaceTimesAndPhotosHtmlFile(RaceData race) {
		File raceTimesFile = new File(race.getRaceFolder(),
				RACE_TIMES_AND_PHOTOS_HTML_FILE_NAME);
		return raceTimesFile;
	}

	public static File getRaceTimesHtmlFile(String raceName) {
		File raceTimesFile = new File(getRaceFolderWithoutCreating(raceName),
				RACE_TIMES_HTML_FILE_NAME);
		return raceTimesFile;
	}

	public static File getRaceTimesAndPhotosHtmlFile(String raceName) {
		File raceTimesFile = new File(getRaceFolderWithoutCreating(raceName),
				RACE_TIMES_AND_PHOTOS_HTML_FILE_NAME);
		return raceTimesFile;
	}

	/**
	 * 
	 * @param raceName
	 * @return html or null if there's a problem
	 */
	public static String getRaceTimesHtmlFromFile(String raceName) {
		String ret = null;
		File file = getRaceTimesHtmlFile(raceName);
		BufferedReader br = null;
		try {
			br = new BufferedReader(new FileReader(file));
			StringBuffer buf = new StringBuffer();
			while (br.ready()) {
				buf.append(br.readLine() + "\n");
			}
			ret = buf.toString();
		} catch (IOException e) {
			Log.e(TAG, "Error finding html for " + raceName, e);
		}
		finally{
			if(br!=null){
				try {
					br.close();
				} catch (IOException e) {
					Log.e(TAG, "Error closing BufferedReader for " + raceName, e);
				}
			}
		}
		return ret;
	}

	/**
	 * Generates files from a race e.g. HTML files for reviewing race in app,
	 * and also parkrun format timer file
	 * 
	 * @param race
	 */
	public static void generateResultFiles(RaceData race,
			PercentHolder progressIndicator) {
		progressIndicator.setPercent(0);
		generateRaceTimesHtmlFile(race);
		progressIndicator.setPercent(33);
		generateRaceTimesAndPhotosHtmlFile(race);
		progressIndicator.setPercent(66);
		generateParkrunTimerFile(race);
		progressIndicator.setPercent(100);
	}

	private static void generateRaceTimesHtmlFile(RaceData race) {
		PrintStream psHtml = null;
		File raceTimesHtml = getRaceTimesHtmlFile(race);
		try {
			psHtml = new PrintStream(new FileOutputStream(raceTimesHtml));
			psHtml.println("<html><head><title>" + race.getRaceName()
					+ "</title>");
			writeCss(psHtml);
			psHtml.println("</head><body>");

			psHtml.println("<h1>Started at " + race.getStartTimeString()
					+ "</h1>");
			psHtml.println("<p>" + race.getRaceName() + "</p>");
			psHtml.println("<table><tr><th>Postion</th><th>Time</th></tr>");

			for (int idx = 1; idx < race.getResults().size(); idx++) {
				psHtml.println("<tr>");
				long time = race.getResults().get(idx).first;
				psHtml.println("<td>" + idx + "</td><td>"
						+ RaceData.getTimeString(time) + "</td>");
				psHtml.println("</tr>");
			}
			psHtml.println("</table>");
			// psHtml.println("<p><a href='" + Utils.ANDROID_APP_URL + "'>" +
			// Utils.ANDROID_APP_URL + "</a></p>");
			psHtml.println("</body>");
			psHtml.println("</html>");
		} catch (IOException e) {
			Log.e(TAG,
					"Couldn't generate html file "
							+ raceTimesHtml.getAbsolutePath(), e);
		} finally {
			if (psHtml != null) {
				psHtml.close();
			}
		}
	}

	private static void generateRaceTimesAndPhotosHtmlFile(RaceData race) {
		PrintStream psHtml = null;
		File raceTimesHtml = getRaceTimesAndPhotosHtmlFile(race);
		try {
			psHtml = new PrintStream(new FileOutputStream(raceTimesHtml));
			psHtml.println("<html><head><title>" + race.getRaceName()
					+ "</title>");
			writeCss(psHtml);
			psHtml.println("</head><body>");
			psHtml.println("<h1>Started at " + race.getStartTimeString()
					+ "</h1>");
			psHtml.println("<p>" + race.getRaceName() + "</p>");
			psHtml.println("<table><tr><th>Postion</th><th>Time</th><th></th></tr>");
			for (int idx = 0; idx < race.getResults().size(); idx++) {
				psHtml.println("<tr>");
				long time = race.getResults().get(idx).first;
				if (idx == 0) {
					psHtml.println("<td>Start</td><td>"
							+ race.getStartTimeString() + "</td>");
				} else {
					psHtml.println("<td>" + idx + "</td><td>"
							+ RaceData.getTimeString(time) + "</td>");
				}
				boolean photoCheck = race.getResults().get(idx).second;
				psHtml.println("<td>");
				if (photoCheck) {
					psHtml.println("<img width='200px' src='" + IMAGES_FOLDER
							+ "/" + Utils.getImageFileName(time) + "' />");
				} else {
					psHtml.println("(No photo)");
				}
				psHtml.println("</td>");
				psHtml.println("</tr>");
			}
			psHtml.println("</table>");
			// psHtml.println("<p><a href='" + Utils.ANDROID_APP_URL + "'>" +
			// Utils.ANDROID_APP_URL + "</a></p>");
			psHtml.println("</body>");
			psHtml.println("</html>");
		} catch (IOException e) {
			Log.e(TAG,
					"Couldn't generate html file "
							+ raceTimesHtml.getAbsolutePath(), e);
		} finally {
			if (psHtml != null) {
				psHtml.close();
			}
		}
	}

	private static void writeCss(PrintStream psHtml) {
		psHtml.println("<style type=\"text/css\">");
		psHtml.println("body{");
		psHtml.println("	font-family: Arial;");
		psHtml.println("	}");
		psHtml.println("table, th, td");
		psHtml.println("{");
		psHtml.println("border-collapse:collapse;");
		psHtml.println("border: 1px solid black;");
		psHtml.println("}");
		psHtml.println("td, th");
		psHtml.println("{");
		psHtml.println("padding:5px;");
		psHtml.println("}");
		psHtml.println("</style>");
	}

	public static byte[] readFile(String file) throws IOException {
		return readFile(new File(file));
	}

	public static byte[] readFile(File file) throws IOException {
		// Open file
		RandomAccessFile f = new RandomAccessFile(file, "r");

		try {
			// Get and check length
			long longlength = f.length();
			int length = (int) longlength;
			if (length != longlength)
				throw new IOException("File size >= 2 GB");

			// Read file and return data
			byte[] data = new byte[length];
			f.readFully(data);
			return data;
		} finally {
			f.close();
		}
	}

	// File name like parkrun_timer_04082012104702.txt with entries as below
	// (using windows line endings)
	// STARTOFEVENT,04/08/2012 09:05:18,020292
	// 0,04/08/2012 09:05:26
	// 1,04/08/2012 09:05:28,00:00:02
	// 2,04/08/2012 09:23:46,00:18:20
	// 3,04/08/2012 09:24:02,00:18:36
	// 4,04/08/2012 09:24:21,00:18:55
	// 5,04/08/2012 09:25:19,00:19:53
	// 6,04/08/2012 09:25:23,00:19:57
	// ...
	// ENDOFEVENT,18/08/2012 10:00:55

	// 3.4 FORMAT OF THE OUTPUT DATA FILES
	// The output data files will be written as a simple text file using comma
	// delimited
	// records in the following format:
	// STARTOFEVENT,[Date/ Time],[SerialNo]
	// 0,[Date/ Time]
	// 1,[Date/ Time],[Time Difference]
	// 2,[Date/ Time],[Time Difference]
	// 3,[Date/ Time],[Time Difference]
	// :
	// :
	// N,[Date/ Time],[Time Difference]
	// ENDOFEVENT,[Date/ Time]
	// where
	// STARTOFEVENT,[Date /Time],[OPN2002 Serial No.] is the Start of Event
	// record created by pressing and holding the OPN2002 trigger button for
	// approx
	// 6 seconds. The record includes the serial no. of the OPN2002
	// 0,[Date/ Time] is the initial time record representing the start of the
	// race. All
	// subsequent timing records will be timed from this record.
	// 1,[Date/ Time],[Time Difference] ... N,[Date/ Time],[Time Difference] are
	// the
	// individual race position records including the time logged against that
	// position
	// and the time difference from the start of Race (Position 0) record.
	// ENDOFEVENT,[Date/ Time] is the End Of event record created by pressing
	// and holding the trigger button on the OPN2002 for approx. 6 seconds.
	//

	public static void generateParkrunTimerFile(RaceData race) {
		File parkrunFile = new File(race.getRaceFolder(), "parkrun_timer.txt");
		String versionText = "Android Photo Stopwatch Version "
				+ getVersionName();
		if (versionText.contains("\"") || versionText.contains(",")
				|| versionText.contains(":") || versionText.contains("'")
				|| versionText.contains("\\") || versionText.contains("/")) {
			Log.e(TAG, "Invalid version text " + versionText);
			versionText = "000000";
		}

		PrintStream ps = null;
		try {
			if (parkrunFile.exists()) {
				parkrunFile.delete();
			}
			ps = new PrintStream(new FileOutputStream(parkrunFile));
			ps.print("STARTOFEVENT,"
					+ parkrunDateTimeFormat.format(race.getStartTimeDate())
					+ "," + versionText);// start of event time would be when
											// opticon
											// timer is primed, serial number of
											// opticon
											// timer
											// // set to 000000 here
			ps.print(WINDOWS_NEWLINE);
			ps.print("0,"
					+ parkrunDateTimeFormat.format(race.getStartTimeDate()));// no
																				// difference
																				// between
																				// start
																				// of
																				// event
																				// time
																				// and
																				// start
																				// time
			ps.print(WINDOWS_NEWLINE);
			for (int position = 1; position < race.getResults().size(); position++) {
				ps.print(position
						+ ","
						+ parkrunDateTimeFormat.format(race
								.getDateForPostion(position))
						+ ","
						+ RaceData.getParkrunTimeString(race
								.getTimeForPosition(position)));
				ps.print(WINDOWS_NEWLINE);
			}
			ps.print("ENDOFEVENT," + parkrunDateTimeFormat.format(new Date()));
			ps.print(WINDOWS_NEWLINE);

		} catch (FileNotFoundException e) {
			Log.e(TAG,
					"Couldn't generate parkrun format file "
							+ parkrunFile.getAbsolutePath(), e);
		} finally {
			if (ps != null) {
				ps.close();
			}
		}
	}

	/**
	 * Sort out camera rotation
	 * 
	 * @param activity
	 * @param cameraId
	 * @param camera
	 */
	@SuppressLint("NewApi")
	public static void setCameraDisplayOrientation(Activity activity,
			int cameraId, android.hardware.Camera camera) {
		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.FROYO) {
			android.hardware.Camera.CameraInfo info = new android.hardware.Camera.CameraInfo();
			android.hardware.Camera.getCameraInfo(cameraId, info);
			int rotation = activity.getWindowManager().getDefaultDisplay()
					.getRotation();
			int degrees = 0;
			switch (rotation) {
			case Surface.ROTATION_0:
				degrees = 0;
				break;
			case Surface.ROTATION_90:
				degrees = 90;
				break;
			case Surface.ROTATION_180:
				degrees = 180;
				break;
			case Surface.ROTATION_270:
				degrees = 270;
				break;
			}

			int result;
			if (info.facing == Camera.CameraInfo.CAMERA_FACING_FRONT) {
				result = (info.orientation + degrees) % 360;
				result = (360 - result) % 360; // compensate the mirror
			} else { // back-facing
				result = (info.orientation - degrees + 360) % 360;
			}
			camera.setDisplayOrientation(result);
		}
	}

	public static String getVersionName() {
		return versionName;
	}

	public static void setVersionName(String versionName) {
		Utils.versionName = versionName;
	}

	/**
	 * http://stackoverflow.com/questions/1399126/java-util-zip-recreating-
	 * directory-structure
	 * 
	 * @param directory
	 * @param zipfile
	 * @throws IOException
	 */
	public static void zip(File directory, File zipfile, PercentHolder progressIndicator) throws IOException {
		int fileCount = countFilesInDirectory(directory);
		URI base = directory.toURI();
		LinkedList<File> queue = new LinkedList<File>();
		queue.addLast(directory);
		ZipOutputStream zout = new ZipOutputStream(new FileOutputStream(zipfile));
		try {
			
			int progressCount = 0;
			while (!queue.isEmpty()) {
				// directory = queue.pop();after api 9
				directory = queue.removeLast();

				for (File kid : directory.listFiles()) {
					progressCount++;
					progressIndicator.setPercent((100*progressCount)/fileCount);
					String name = base.relativize(kid.toURI()).getPath();
					if (kid.isDirectory()) {
						queue.addLast(kid);
						name = name.endsWith("/") ? name : name + "/";
						zout.putNextEntry(new ZipEntry(name));
					} else {
						zout.putNextEntry(new ZipEntry(name));
						copy(kid, zout);
						zout.closeEntry();
					}
				}
			}
		} finally {
			zout.close();
		}
	}

	 /**
	   * Count files in a directory (including files in all subdirectories)
	   * @param directory the directory to start in
	   * @return the total number of files
	   */
	  public static int countFilesInDirectory(File directory) {
	      int count = 0;
	      for (File file : directory.listFiles()) {
	          if (file.isFile()) {
	              count++;
	          }
	          if (file.isDirectory()) {
	              count += countFilesInDirectory(file);
	          }
	      }
	      return count;
	  }
	
	
	private static void copy(InputStream in, OutputStream out)
			throws IOException {
		byte[] buffer = new byte[1024];
		while (true) {
			int readCount = in.read(buffer);
			if (readCount < 0) {
				break;
			}
			out.write(buffer, 0, readCount);
		}
	}

	private static void copy(File file, OutputStream out) throws IOException {
		InputStream in = new FileInputStream(file);
		try {
			copy(in, out);
		} finally {
			in.close();
		}
	}

}
